// $Id: ConnMgr.cc,v 1.9.2.2 1998/12/15 07:27:56 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Connection.h"
#include "Xaction.h"
#include "ConnMgr.h"
#include "Log.h"


/* ConnMgrCfg */

ConnMgrCfg::ConnMgrCfg():
	theMaxCcUseLevel(-1), theMaxTotUseCount(-1), theTargetCcLevel(-1) {	
}

/* ConnMgr */

ConnMgr::~ConnMgr() {
	for (int i = 0; i < theConns.count(); ++i)
		delete theConns[i];
	theConns.reset();
}


ConnMgr *ConnMgr::clone() const {
	ConnMgr *mgr = new ConnMgr(0);
	mgr->configure(theCfg);
	return mgr;
}

ConnInfo *ConnMgr::get(const Address &addr) {
	Notepad << "getting conn for " << addr << " max count: " << theConns.count() << ende;
	ConnInfo *ci = findReusable(addr);
	return ci ? use(ci) : use(open(addr));
}

void ConnMgr::put(ResMgr *mgr) {
	if (!mgr || !exists(mgr->resIdx(), mgr->resId()))
		return;
	ConnInfo *ci = theConns[mgr->resIdx()];
	Notepad << "puting connection " << ci << " max count: " << theConns.count() << ende;
	Notepad << "connection " << ci << " usable: " << usable(ci) << " idle: " << ci->idle() << " reusable: " << reusable(ci) << ende;
	if (!usable(ci))
		close(ci);
	else
	if (ci->idle() && !reusable(ci))
		close(ci);
}

void ConnMgr::del(ResMgr *mgr) {
	if (!mgr || !exists(mgr->resIdx(), mgr->resId()))
		return;
	Notepad << "deling connection " << theConns[mgr->resIdx()] << " max count: " << theConns.count() << ende;
	close(theConns[mgr->resIdx()]);
}

ConnInfo *ConnMgr::use(ConnInfo *ci) {
	if (!usable(ci) || !reusable(ci)) {
		close(ci);
		delete ci;
		return 0;
	}
	ci->use();
	return ci;
}

void ConnMgr::close(ConnInfo *ci) {
	if (ci) {
		const Address addr = ci->conn()->addr();
		theConns[ci->idx()] = 0;
		assert(ci->conn());
		ci->conn()->close();
		delete ci;
		theCcLevel--;
		assert(theCcLevel >= 0);
		if (theCcLevel < theCfg.theTargetCcLevel)
			open(addr);
	}
}

bool ConnMgr::exists(int idx, int id) const {
	return 
		idx < theConns.count() &&
		theConns[idx] &&
		theConns[idx]->id() == id;
}

ConnInfo *ConnMgr::open(const Address &addr) {
	Connection *conn = new Connection(addr);
	if (!conn->open()) {
		delete conn;
		return 0;
	}
	theCcLevel++;
	return add(conn);
}

ConnInfo *ConnMgr::add(Connection *conn) {
	assert(conn);
	assert(theOwner);
	ConnInfo *ci = new ConnInfo(theOwner, conn);
	int idx;
	if (theConns.find(0, idx))
		theConns[idx] = ci;
	else
		theConns.append(ci);
	ci->idx(idx);
	return ci;
}

// returns existing connection if any
ConnInfo *ConnMgr::findReusable(const Address &addr) {
	ConnInfo *best_ci = 0;
	int best_level = -1;

	for (int i = 0; i < theConns.count(); ++i) {
		ConnInfo *ci = theConns[i];
		if (!ci || !ci->conn())
			continue;
		if (ci->conn()->addr() != addr)
			continue;
		if (!reusable(ci))
			continue;
		const int lock_level = max(
			ci->resMgr(ConnInfo::dirIn)->queue()->length(),
			ci->resMgr(ConnInfo::dirOut)->queue()->length());
		if (canShare(ci, lock_level)) {
			if (!best_ci || best_level > lock_level) {
				best_ci = ci;
				best_level = lock_level;
			}
		}
	}
	return best_ci;
}
