// $Id: RndRepMaker.cc,v 1.25.2.8 1998/12/03 01:34:13 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "IOBuffer.h"
#include "RndRepMaker.h"
#include "Rnd.h"
#include "Pause.h"
#include "LocalStats.h"
#include "Log.h"
#include "Gadgets.h"
#include "globals.h"

#include "Xaction.h" /* for updating theRespSize */

Size RndRepMaker::calcRepSize() const {
	if (TheOpt.theRepSzDistr) {
		// XXX: hard coded exponential distribution by default
		static ExpDistr TheSizeDistr(new RndGen(), TheWorld.objSizeMean());
	
		const int objId = theCont.theReqHdr->objId();
		Assert(objId > 0 && objId <= TheWorld.capacity());
		// seed random generator with objId
		TheSizeDistr.rndGen().seed((unsigned)objId);
		Size sz = (size_t) TheSizeDistr();
		// XXX: hard coded size limits:
		// most popular objects should not be too big (100*id + 10KB)
		// objects should not exceed 50MB
		sz %= min((Size)(100*objId + Size::KB(10)), Size::MB(50));
		// the object must be at least CBlock size
		return max(sz, (Size)32);
	} else {
		// constant distribution
		return TheWorld.objSizeMean();
	}
}

void RndRepMaker::make200Prefix() {

	// determine the size of the reply
	const Size csz = calcRepSize();
	theCBlock.contSize(csz);
	
	// put HTTP reply headers
	IOBuffer &buf = theConn->theWriterBuf;
	const int xid = theCont.theReqHdr->xactId() > 0 ?
		theCont.theReqHdr->xactId() + 2 : -1;
	Assert(xid > 0);
	buf	<< "HTTP/1.0 200" << crlf; // 1.1
	buf << "Content-Length: " << (int)csz << crlf;
//	buf << "Expires: Fri, 16 Oct 1998 02:41:52 GMT" << crlf;

	if (TheOpt.isRepCachable)
		buf << "Cache-Control: public,max-age=31536000" << crlf;
	else
		buf << "Cache-Control: private,no-cache" << crlf;

	if (xid > 0)
		buf << "X-Xact: " << xid << crlf;
	buf << crlf;

	maxOffset(workOffset() + csz);
	Notepad << this << " 200 rep; cont size:" << csz << " total: " << sizeTotal() << " buf: " << buf.contSize() << ende;

	// put CBlock
	buf << theCBlock;

	// XXX: big kludge
	((Xaction*)parent()->parent())->theRespSize = csz;

}

void RndRepMaker::make304Prefix() {

	theCBlock.contSize(0);

	// put HTTP reply headers
	IOBuffer &buf = theConn->theWriterBuf;
	buf	<< "HTTP/1.0 304" << crlf // 1.1
		<< "Cache-Control: public,max-age=31536000" << crlf
		<< crlf;

	maxOffsetHere();

	StatInc(the304SentCount);
	Notepad << this << " 304 rep; size total: " << sizeTotal() << " buf: " << buf.contSize() << ende;
}

// append random content to the io buffer
void RndRepMaker::doContent() {
	Assert(theOtherContState == csValid);

	Notepad << "sleep state: " << theSleepState << " think time: " << TheOpt.theSrvWriteDelay << ende;
	// sleep for a while if needed (KLUDGE)
	if (theSleepState == ssSleeping)
		return;
	if (theSleepState == ssAwake && TheOpt.theSrvWriteDelay >= 0) {
		static RndPause TheSleepPause(new ExpDistr(new RndGen(1), TheOpt.theSrvWriteDelay.secd()));
		const Time st = TheSleepPause.pause();
		Notepad << "sleeping for " << st << ende;
		theSleepState = ssSleeping;
		sleepFor(st);
		return;
	}
	theSleepState = ssAwake;

	IOBuffer &buf = theConn->theWriterBuf;

	if (theCBlock.contSize() < 0)
		theCont.theReqHdr->ims() ? make304Prefix() : make200Prefix();

	const Size size_app(min(buf.bufSize(), sizeLeft()));
	if (size_app > 0)
		buf.appended(size_app); // just use buffer area, no content

	// check if we want to close the connection
	if (thePersFlag && sizeLeft() <= 0)
		buf.eod(true);
}
