#ifndef _CLIENT_H_
#define _CLIENT_H_ "$Id: Client.h,v 1.13.2.1 1998/09/22 07:43:58 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AlarmClock.h"
#include "Xaction.h"
#include "ConnInfo.h"
#include "Loader.h"
#include "TopProcess.h"
#include "Array.h"

class ConnMgr;
class ProgressMeter;
class AddrGen;

// just to simplify configuration interface
class ClientCfg {
	public:
		ClientCfg();
		~ClientCfg() {} // does not destroy!

		ClientCfg clone() const;
		void destroy();

	public:
		ConnMgr *theConnMgr;
		ProgressMeter *theProgressMeter;
		Loader *theLoader;
		AddrGen *theAddrGen;

		XactCfg theXactCfg;
};

// emulates a Web client
class Client: public TopProcess, public Loadee, public ResOwner {
	public:
		Client(): theLaunchDebt(0) {}
		virtual ~Client();

		virtual Thread *clone() const;

		void configure(const ClientCfg &aCfg);

		virtual void taskStarted(Thread *);
		virtual void taskStopped(Thread *);

		virtual void idleRes(ResMgr *);
		virtual void voidRes(ResMgr *);
		virtual void changeLoad(const LoadInfo &);

		virtual void checkProgress();
		virtual void init();
		virtual void run();
		virtual void removed();

		virtual const char *name() const { return "Client"; }

	protected:
		virtual void beIdle();

		bool launchXact(const Address &addr); // with a given address
		Xaction *genXaction(ConnInfo &ci);

		ConnMgr *connMgr() { return theCfg.theConnMgr; }

	protected:
		ClientCfg theCfg;
		int theLaunchDebt;
};

#endif
