#ifndef _CONN_MGR_H_
#define _CONN_MGR_H_ "$Id: ConnMgr.h,v 1.9.2.2 1998/12/15 07:28:03 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Array.h"
#include "Address.h"
#include "Socket.h"
#include "ConnInfo.h"

// coniguration record for a conn mgr
class ConnMgrCfg {
	public:
		ConnMgrCfg();

	public:
		int theMaxCcUseLevel;   // max number of cc request per connection
		int theMaxTotUseCount;  // max for the total number a conn can be used
		int theTargetCcLevel;   // #cc connections to keep ready
};

// manages a collection of network connections
class ConnMgr {
	public:
		ConnMgr(ResOwner *anOwner): theOwner(anOwner) {}
		virtual ~ConnMgr();

		void configure(const ConnMgrCfg &aCfg) { theCfg = aCfg; }

		virtual ConnMgr *clone() const;
		void owner(ResOwner *anOwner) { theOwner = anOwner; }

		virtual ConnInfo *get(const Address &anAddr);
		virtual void put(ResMgr *mgr);
		virtual void del(ResMgr *mgr);

	protected:
		bool exists(int idx, int id) const;
		bool usable(ConnInfo *ci) const { return ci && ci->good() && !ci->exhausted(); }
		bool reusable(ConnInfo *ci) const { return ci && (theCfg.theMaxTotUseCount < 0 || ci->useCount() < theCfg.theMaxTotUseCount); }

		ConnInfo *use(ConnInfo *ci);
		ConnInfo *add(Connection *conn);

		ConnInfo *open(const Address &addr);
		void close(ConnInfo *ci);

		virtual ConnInfo *findReusable(const Address &addr);
		virtual bool canShare(ConnInfo *, int curCcLevel) { return theCfg.theMaxCcUseLevel < 0 || curCcLevel < theCfg.theMaxCcUseLevel; }

	protected:
		ResOwner *theOwner;
		SchArray<ConnInfo*> theConns;
		Array<ConnInfo*> theReadyUnusedConns;
		int theCcLevel;       // number of ready connections (used or not)

		ConnMgrCfg theCfg;
};

#endif
