#ifndef _CONNECTION_H_
#define _CONNECTION_H_ "$Id: Connection.h,v 1.12.2.5 1998/11/25 18:16:28 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "IOBuffer.h"
#include "Socket.h"
#include "Address.h"
#include "SelectLoop.h" /* for IODir declaration */
#include "SafePtr.h"

class XactRec;

// a bi-directional connection with read/write buffers/locks
class Connection {
	public:
		Connection(): theLocPort(-1), theState(csNone) { theIOReadyFlags[ioRead] = theIOReadyFlags[ioWrite] = theIOReadyFlags[ioExcept] = false; }
		Connection(const Address &anAddr): theAddr(anAddr), theLocPort(-1), theState(csNone) { theIOReadyFlags[ioRead] = theIOReadyFlags[ioWrite] = theIOReadyFlags[ioExcept] = false; }
		~Connection() {}

		const Address &addr() const { return theAddr; }
		Socket socket() { return theSocket; }
		bool good() const { return theReaderBuf.good() && theWriterBuf.good(); }
		bool exhausted() const { return theReaderBuf.exhausted() || theWriterBuf.exhausted(); }
		bool ioReady(IODir dir) const { return theIOReadyFlags[dir]; }
		bool connected() const { return theState > csPending; }

		void addr(const Address &anAddr) { theAddr = anAddr; }
		void setIOReady(IODir dir) { theIOReadyFlags[dir] = true; }

		bool open(); // requires the addr to be set
		bool accept(Socket listenSock, bool &fatal); // sets the addr
		Size read(Size max_size);  // from the socket into the rBuf
		Size write(Size max_size); // from the wBuf into the socket
		void close(); // requires that no locks are set

	public:
		IOBuffer theReaderBuf;
		IOBuffer theWriterBuf;

	protected:
		Socket theSocket;
		Address theAddr;  // remote address
		int theLocPort;   // local port for connect()-ed connections 
		Time theStart;

		enum { csNone, csPending, csPumping, csClosed } theState;
		bool theIOReadyFlags[3]; // 3 == countof(IODir)
};

#endif
