#ifndef _LOAD_GEN_H_
#define _LOAD_GEN_H_ "$Id: LoadGen.h,v 1.2.2.2 1998/11/14 23:28:39 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Loader.h"
#include "Pause.h"


// Load generator configuration
class LoadGenCfg {
	public:
		LoadGenCfg();

		void destroy();
		LoadGenCfg clone() const;

	public:
		int theBlobSize;        // number of events per blob
		int theBlobCount;       // number of blobs to generate
		Pause *theIntraBlobPause; // time between events in a blob
		Pause *theInterBlobPause; // time between blobs
};

// LoadGet is a Loader that emits changeLoad events in "blobs"
class LoadGen: public Loader {
	public:
		LoadGen(): theBlobSize(0), theBlobCount(0) {}
		~LoadGen() { theCfg.destroy(); }

		void configure(const LoadGenCfg &aCfg) { theCfg = aCfg; }
		virtual Loader *clone() const;

	protected:
		virtual bool enough() const { return Loader::enough() || theCfg.theBlobCount >= 0 && theBlobCount >= theCfg.theBlobCount; }
		bool blobDone() const { return theBlobSize >= theCfg.theBlobSize; }

		virtual void load();

	protected:
		LoadGenCfg theCfg;
		int theBlobSize;  // events in the current blob
		int theBlobCount; // blobs generated so far
};

// attempts to support a given, constant load (steady stream)
class StreamLoader: public Loader {
	public:
		StreamLoader(int aLevel): theLevel(aLevel) {}

		virtual Loader *clone() const { return new StreamLoader(theLevel); }

		virtual void loadChange(const LoadInfo &li);

	protected:
		virtual void load() { changeLoad(LoadInfo::Level(theLevel)); }

	protected:
		int theLevel;
};

#endif
