// $Id: PhaseMgr.h,v 1.1.2.4 1998/11/08 04:56:43 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _PHASE_MGR_H_
#define _PHASE_MGR_H_


// #include "RateMeter.h"
#include "LocalStats.h"

#include <iostream.h>

// maintains info about one metering phase
class Phase {
	public:
		Phase(const char *aName = "");
		~Phase();

		void name(const char *aName) { theName = aName; }

		bool enough() const;
		bool started() const { return theStartTime > 0; }
		bool stopped() const { return theStopTime > 0; }
		const char *name() const { return theName; }

		void goal(int absComplCount) { theGoal = absComplCount; }
		void incGoal(int delta);

		void start(LocalStats *&stats);
		void stop();
		void reset();

		void report(ostream &os, bool full = false) const;

	protected:
		int submCount() const { return theStats->theSubmMeter.count(); }
		int complCount() const { return theStats->theComplMeter.count(); }

		void reportRespTimeAndSize(ostream &os) const;

	protected:
		const char *theName;
		LocalStats *theStats;
		Time theStartTime;
		Time theStopTime;
		int theGoal;
};

// maintains phases
class PhaseMgr {
	public:
		PhaseMgr();
		~PhaseMgr();

		void start();
		void lastReport();

		void checkPhase();

	protected:
		void nextPhase();

	protected:
		Phase *thePhase;
		Phase *thePinger;
		enum RunState { rsNone = -1, rsWarmUp = 0, rsMeasure, rsCoolOff } theState;
};

#endif
