// $Id: Ring.h,v 1.1.2.4 1998/11/25 19:42:21 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef RING_CLASS_H
#define RING_CLASS_H

#include "AppDef.h"

#include "Array.h"

// FIFO Queue of <Item>s with fixed capacity (i.e., ring)

template <class Item>
class Ring: protected Array<Item> {
	public:
		Ring(int aCapacity = 0): Array<Item>(aCapacity), theInOff(0), theOutOff(0) {}

		int count() const { return theInOff - theOutOff; }
		bool empty() const { return theInOff <= theOutOff; }

		void enqueue(Item i) { item(theInOff++ % theCapacity) = i; check(); }
		Item dequeue() { return item(theOutOff++ % theCapacity); }

		void resize(int aCap) { Assert(empty()); theCount = theInOff = theOutOff = 0; grow(aCap); }

	protected:
		inline void check();

	protected:
		int theInOff;
		int theOutOff;
};

/* implementaion of in-lined methods */

template <class Item>
inline
void Ring<Item>::check() {
	Assert(theInOff > theOutOff && count() <= theCapacity);
	if (theOutOff >= theCapacity) {
		theInOff %= theCapacity;
		theOutOff %= theCapacity;
		if (theInOff <= theOutOff)
			theInOff += theCapacity;
	}
}

#endif
