#ifndef _SOCKET_CLASS_H_
#define _SOCKET_CLASS_H_ "$Id: Socket.h,v 1.11.2.5 1998/12/15 07:28:05 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include "Address.h"

#include <sys/types.h>
#include <sys/socket.h>

// UNIX socket wrapper
class Socket {
	public:
		static bool IsEnabled; // bind,connect,listen,read,write calls are enabled

	public:
		Socket(): theFD(-1) {}
		Socket(int aFD): theFD(aFD) {}
		~Socket() {}

		bool create();
		bool create(int domain, int type, int protocol);
		bool connect(const char *host, int port);
		bool bind(int port);
		bool listen(int port);
		Socket accept(struct sockaddr *addr, int *addr_len);
		ssize_t read(void *buf, size_t sz);
		ssize_t write(const void *buf, size_t sz);
		bool close();

		// XXX: these should return bool
		void reuseAddr(bool set) { setOpt(SO_REUSEADDR, set); }
		void reusePort(bool set);
		void linger(int delay);
		void blocking(bool set);
		void tcpDelay(bool set);
		void sendBuf(int size);
		void recvBuf(int size);

		bool isOpen() const { return theFD >= 0; }
		bool isBlocking() const;
		int flags() const;
		int lport() const;
		int rport() const;
		struct timeval rcvTimeout() const;
		int sendBuf() const;
		int recvBuf() const;
		int errNo() const;
		bool peerAddr(struct sockaddr *addr, int &len) const;
		Address peerAddr() const;

		operator int() const { return theFD; }
		Socket &operator =(int aFD) { theFD = aFD; return *this; }

	protected:
		bool setOpt(int optName, bool set);
		bool setOpt(int optName, const void *optVal, int optLen);
		bool getOpt(int optName, void *optVal, int *optLen) const;

	protected:
		int theFD;
};

#endif
