#ifndef _THREAD_CLASS_H_
#define _THREAD_CLASS_H_ "$Id: Thread.h,v 1.12.2.1 1998/09/22 07:44:30 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "UseAcc.h"
#include "Identity.h"
#include "Kernel.h"

#include <iostream.h>


class Process;

// a thread is a seudo-concurrent activity
// threads are spawned by processes (owners)
// a thread informs the owner about thread's progress
// an owner may abort an execution of a thread

class Thread: public virtual Identity, public virtual SelfAcc {
	public:
		enum State { thUnkown = 0, thStarting, thRunnable, thSleeping, thDone };

	public:
		Thread(): theParent(0), theKernIdx(-1), theActIdx(-1), theState(thStarting) {}
		virtual ~Thread() {}

		virtual Thread *clone() const = 0;

		int kernIdx() const { return theKernIdx; }
		int actIdx() const { return theActIdx; }
		const Process *parent() const { return theParent; }
		Process *parent() { return theParent; }

		bool active() const { return theActIdx >= 0; }
		bool runnable() const { return theState == thRunnable; }
		bool done() const { return theState == thDone; }

		void kernIdx(int anIdx) { theKernIdx = anIdx; }
		void actIdx(int anIdx) { theActIdx = anIdx; }
		void parent(Process *aParent) { theParent = aParent; }
		
		/* called by kernel only */
		virtual void checkProgress() = 0;
		virtual void init() = 0;  // called on start, before first run
		virtual void run() = 0;
		virtual void removed() {} // called before Kernel deletes us
		
		virtual ostream &printId(ostream &os) const;

	protected:
		void beActive() { if (!active()) TheKernel.activate(this); }
		void state(State aState) { if (!done()) { theState = aState; if (done()) beGone(); } }
		void beActive(State aState) { state(aState); beActive(); }

	protected:
		Process *theParent;
		int theKernIdx;      // idx for kernel registration table
		int theActIdx;       // idx for "active threads" kernel table
		State theState;
};


#include "Log.h"

// a ``thread signal'' macro
#define thSignal(thread, method) { \
	Notepad << this << ": " << thread << "->" << #method << "()" << ende; \
	if (thread && !thread->done()) thread->method(); \
}

// a ``thread rpc'' macro
#define thCall(thread, method, param) { \
	Notepad << this << ": " << thread << "->" << #method << '(' << param << ')' << ende; \
	if (thread && !thread->done()) thread->method(param); \
}

#endif
