// $Id: Trace.h,v 1.1.2.2 1998/09/22 07:44:33 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _TRACE_H_
#define _TRACE_H_

#include "Stats.h"
#include "Array.h"

#include <iostream.h>

// a simple fixed window size trace of integer values
// with some statistical capabilities
class Trace {
	public:
		Trace(const char *aLabel, const char *aMeasure, int aWinCount);
		~Trace() { delete[] theVals; }

		const SampleStat &stat() const { return theStat; }
		int keptCount() const;

		void record(int n) { theVals[theCount++ % theWinCount] = n; theStat.record(n); }

		void perc(Array<int> &perc) const; // returns percentiles
		void report(ostream &os) const; // prints a report

		int item(int idx) const { return theVals[idx]; }
		int operator [](int idx) const { return item(idx); }

	protected:
		SampleStat theStat;
		int theWinCount;    // max number of recent events kept (window "size")
		int theCount;       // number of events recorded
		int *theVals;       // window (event values)

		const char *theLabel;   // trace name to use in reports
		const char *theMeasure; // measure of values (e.g. "second")
};


#endif
